@@ -1,125 +1,143 @@
 package DTL::Fast;
 use strict; use warnings FATAL => 'all'; 
 use parent 'Exporter';
-use Carp qw(confess);
+use Carp;
+use Digest::MD5 qw(md5_hex);
 
-use 5.018002;
-our $VERSION = '1.00';
+use 5.010;
+our $VERSION = '1.05.1';
+
+require XSLoader;
+XSLoader::load('DTL::Fast', $VERSION);
 
-use Cwd;
 use DTL::Fast::Template;
+use DTL::Fast::Cache::Runtime;
+use DTL::Fast::Cache::Serialized;
+
+our $RUNTIME_CACHE;
+our $SERIALIZED_CACHE;
 
 our @EXPORT_OK;
 
 # @todo Texts should be tossed by references
-# @todo Protection from cycled ierarchy
 # @todo These should be done via Cache class which can work with last modified date
-our %TEMPLATES_CACHE = ();
-our $TEMPLATES_CACHE_HITS = 0;
-our %OBJECTS_CACHE = ();
-our $OBJECTS_CACHE_HITS = 0;
 
 push @EXPORT_OK, 'get_template';
 sub get_template
 {
-    my $template_name = shift;
-    my $dirs = shift // [getcwd()];
-    my %kwargs = @_;
+    my( $template_name, %kwargs ) = @_;
     
-    confess "Template name was not specified" 
+    croak  "Template name was not specified" 
         if not $template_name;
-    
-    confess "Second parameter must be a dirs array reference" 
-        if(
-            not ref $dirs
-            or ref $dirs ne 'ARRAY'
-            or not scalar @$dirs
-        );
 
-    my $cache_key = sprintf '%s:%s:%s:%s'
-        , __PACKAGE__
-        , $template_name
-        , join( ',', @$dirs )
-        , join( ',', @{$kwargs{'ssi_dirs'}//[]})
+    croak "Template directories array was not specified"
+        if
+            not defined $kwargs{'dirs'}
+            or ref $kwargs{'dirs'} ne 'ARRAY'
+            or not scalar @{$kwargs{'dirs'}}
         ;
 
+    my $cache_key = _get_cache_key( $template_name, %kwargs );
+
     my $template;
+
+    $RUNTIME_CACHE //= DTL::Fast::Cache::Runtime->new();
     
-    if( exists $OBJECTS_CACHE{$cache_key} ) # already has object with this source parameters
+    if( 
+        $kwargs{'no_cache'}
+        or not defined ( $template = $RUNTIME_CACHE->get($cache_key))
+    )
     {
-        $template = $OBJECTS_CACHE{$cache_key};
-        $OBJECTS_CACHE_HITS++;
-    }    
-    else
-    {
-        $template = _read_template($template_name, $dirs);
-        $template =~ s/\{\% (?:block|endblock|extends) .*?\%\}//gs;
+        $template = read_template($template_name, %kwargs );
         
-        my @arguments = ($template, $dirs);
-        push @arguments, 'ssi_dirs', $kwargs{'ssi_dirs'}
-            if $kwargs{'ssi_dirs'};
-        push @arguments, 'url_source', $kwargs{'url_source'}
-            if $kwargs{'url_source'};
-        
-        $template = DTL::Fast::Template->new(@arguments)
-            if defined $template;
-        $OBJECTS_CACHE{$cache_key} = $template;
-    }
-    
-    return $template;
-}
-
-sub _apply_inheritance
-{
-    my $template = shift;
-    my $dirs = shift;
-    
-    if( $template =~ s/\s*\{\% extends\s*"(.+?)" \%\}//s ) # template has inheritance
-    {
-        my $parent_template = _read_template($1, $dirs);
-        
-        my %named_blocks = (
-            $template =~ /\{\% block ([^\s]+) \%\}(.+?)\{\% endblock \%\}/gs
-        );
-        
-        my $block_names = join '|', keys(%named_blocks);
-        $parent_template =~ s/\{\% block ($block_names) \%\}.+?\{\% endblock \%\}/\{\% block $1 \%\}$named_blocks{$1}\{\% endblock \%\}/gsi;
-        $template = $parent_template;
-    }
-    
-    return $template;
-}
-
-sub _read_template
-{
-    my $template_name = shift;
-    my $dirs = shift;
-
-    my $template = undef;
-
-    my $cache_key = sprintf '%s:%s:%s'
-        , __PACKAGE__
-        , $template_name
-        , join ',', @$dirs;
-
-    if( exists $TEMPLATES_CACHE{$cache_key} )   # already read template with this parameters
-    {
-        $template = $TEMPLATES_CACHE{$cache_key};
-        $TEMPLATES_CACHE_HITS++;
-    }
-    else
-    {
-        $template = _read_file($template_name, $dirs);
-        $template = _apply_inheritance($template, $dirs)
-            if defined $template;
-            
-        $TEMPLATES_CACHE{$cache_key} = $template;
-    }
-
-    confess sprintf( <<'_EOT_', $template_name, join("\n", @$dirs)) if not defined $template;
+        if( defined $template )
+        {
+            $RUNTIME_CACHE->put($cache_key, $template);
+        }
+        else
+        {
+            croak  sprintf( <<'_EOT_', $template_name, join("\n", @{$kwargs{'dirs'}}));
 Unable to find template %s in directories: 
 %s
 _EOT_
+        }
+    }
+   
+    return $template;
+}
+
+sub _get_cache_key
+{
+    my( $template_name, %kwargs ) = @_;
+    
+    return md5_hex(
+        sprintf( '%s:%s:%s:%s'
+            , __PACKAGE__
+            , $template_name
+            , join( ',', @{$kwargs{'dirs'}} )
+            , join( ',', @{$kwargs{'ssi_dirs'}//[]})
+            # shouldn't we pass uri_handler here?
+        )
+    )
+    ;
+}
+
+sub read_template
+{
+    my( $template_name, %kwargs ) = @_;
+    
+    my $template = undef;
+    my $template_path = undef;
+
+    croak "Template directories array was not specified"
+        if not defined $kwargs{'dirs'}
+            or not ref $kwargs{'dirs'}
+            or not scalar @{$kwargs{'dirs'}}
+        ;
+    
+    my $cache_key = _get_cache_key( $template_name, %kwargs );
+
+    $SERIALIZED_CACHE //= DTL::Fast::Cache::Serialized->new();
+    
+    if( 
+        $kwargs{'no_cache'}
+        or not defined ( $template = $SERIALIZED_CACHE->get($cache_key))   # runtime serialized cache reading
+    )
+    {
+        if( 
+            $kwargs{'no_cache'}
+            or not exists $kwargs{'cache'} 
+            or not $kwargs{'cache'}
+            or not $kwargs{'cache'}->isa('DTL::Fast::Cache')
+            or not defined ($template = $kwargs{'cache'}->get($cache_key))
+        )
+        {
+            ($template, $template_path) = _read_file($template_name, $kwargs{'dirs'});
+            
+            if( defined $template )
+            {
+                $kwargs{'file_path'} = $template_path;
+                $template = DTL::Fast::Template->new( $template, %kwargs);
+        
+                $kwargs{'cache'}->put( $cache_key, $template )
+                    if 
+                        defined $template
+                        and exists $kwargs{'cache'}
+                        and $kwargs{'cache'}
+                        and $kwargs{'cache'}->isa('DTL::Fast::Cache')
+                    ;
+            }
+        }
+        
+        $SERIALIZED_CACHE->put($cache_key, $template)
+            if defined $template;
+    }
+    
+    if( defined $template )
+    {
+        $template->{'cache'} = $kwargs{'cache'} if $kwargs{'cache'};
+        $template->{'url_source'} = $kwargs{'url_source'} if $kwargs{'url_source'};
+    }
     
     return $template;
 }
@@ -129,26 +147,28 @@ sub _read_file
     my $template_name = shift;
     my $dirs = shift;
     my $template;
-
+    my $template_path;
+    
     foreach my $dir (@$dirs)
     {
-        $dir =~ s/[\/\\]+$//gsi;
-        my $template_path = sprintf '%s/%s', $dir, $template_name;
+        $dir =~ s/[\/\\]+$//xgsi;
+        $template_path = sprintf '%s/%s', $dir, $template_name;
         if( 
             -e $template_path
             and -f $template_path
             and -r $template_path
         )
         {
-            if( open IF, '<', $template_path )
+            if( open my $IF, '<', $template_path )
             {
-                $template = join '', <IF>;
-                close IF;
+                local $/ = undef;
+                $template = <$IF>;
+                close $IF;
                 last;
             }
             else
             {
-                confess sprintf(
+                croak  sprintf(
                     'Error opening file %s, %s'
                     , $template_path
                     , $!
@@ -157,17 +177,16 @@ sub _read_file
         }        
     }
     
-    return $template;
+    return ($template, $template_path);
 }
 
 # result should be cached with full list of params
 push @EXPORT_OK, 'select_template';
 sub select_template
 {
-    my $template_names = shift;
-    my $dirs = shift // [getcwd()];
+    my( $template_names, %kwargs ) = @_;
     
-    confess "First parameter must be a template names array reference" 
+    croak  "First parameter must be a template names array reference" 
         if(
             not ref $template_names
             or ref $template_names ne 'ARRAY'
@@ -178,7 +197,7 @@ sub select_template
     
     foreach my $template_name (@$template_names)
     {
-        if( ref ( $result = get_template( $template_name, $dirs )) eq 'DTL::Fast::Template' )
+        if( ref ( $result = get_template( $template_name, %kwargs )) eq 'DTL::Fast::Template' )
         {
             last;
         }
@@ -196,7 +215,7 @@ DTL::Fast - Perl implementation of Django templating language.
 
 =head1 VERSION
 
-Version 1.00
+Version 1.05.1
 
 =head1 SYNOPSIS
 
@@ -255,7 +274,7 @@ Using DTL::Fast::Template constructor:
     
     my $tpl = DTL::Fast::Template->new(
         $template_text,                             # template itself
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # optional, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -266,7 +285,7 @@ Using DTL::Fast::Template constructor:
     
     my $tpl = get_template(
         $template_path,                             # path to the template, relative to directories from second argument
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -279,7 +298,7 @@ when you are using C<get_template> helper function, framework will try to find t
     
     my $tpl = select_template(
         [ $template_path1, $template_path2, ...],   # paths to templates, relative to directories from second argument
-        [ $dir1, $dir2, ... ],                      # optional, directories list to look for parent templates and includes
+        'dirs' => [ $dir1, $dir2, ... ],            # mandatory, directories list to look for parent templates and includes
         'ssi_dirs' => [ $ssi_dir1, $ssi_dir1, ...]  # optional, directories list to look for files included with ssi tag
         'url_source' => \&uri_getter                # optional, reference to a function, that can return url template by model name (necessary for url tag)
     );
@@ -294,7 +313,7 @@ After parsing template using one of the methods above, you may render it using c
     
     my $tpl = get_template(
         'hello_template.txt',          
-        [ '/srv/wwww/templates/' ]
+        'dirs' => [ '/srv/wwww/templates/' ]
     );
     
     print $tpl->render({ name => 'Alex' });
@@ -307,7 +326,7 @@ or
     
     my $tpl = get_template(
         'hello_template.txt',          
-        [ '/srv/wwww/templates/' ]
+        'dirs' => [ '/srv/wwww/templates/' ]
     );
     
     my $context = DTL::Fast::Context->new({
@@ -320,6 +339,19 @@ or
 
     $context->set('name' => 'Sergey');
     print $tpl->render($context);
+
+=head2 Cache classes
+
+To do...
+
+=head2 Custom tags
+
+To do...
+
+=head2 Custom filters
+
+To do...
+
     
 =head1 TEMPLATING LANGUAGE
 
@@ -343,8 +375,6 @@ This module supports all built-in filters documented on L<official Django site|h
 
 =over
 
-=item * Date and time formatting being done using L<C<Date::Format>> module, which uses C library routines C<strftime> and C<ctime>. Django uses it's own L<placeholders|https://docs.djangoproject.com/en/1.7/ref/templates/builtins/#date>. (Mapping considered atm.)
-
 =item * C<ssi> tag in Django uses absolute paths and C<ALLOWED_INCLUDE_ROOTS> configuration option. This library works separately and may be used with different frameworks. So, C<ssi> tag uses relative paths and you MUST specify additional template constructor parameter: C<ssi_dirs> which should be an array reference with list of dirs to search in.
 
 =item * C<csrf_token> tag is not implemented, too well connected with Django.
@@ -375,6 +405,8 @@ May be some of this features implemented in Django itself. Let me know about it.
 
 =over
 
+=item * C<strftime> - new filter. Formatting time using L<C<Date::Format>> module, which is using C functions C<strftime> and C<ctime>.
+
 =item * C<firstofdefined> - new tag, that works like C<firstof> tag, but checks if value is defined (not true)
 
 =item * C<defined> logical operator. In logical constructions you may use C<defined> operator, which works exactly like perl's C<defined>
@@ -403,6 +435,8 @@ is being rendered like:
 
 =over
 
+=item * C<as_bool>           - returns logical representation of object
+
 =item * C<and(operand)>      - makes logical `and` between object and operand
 
 =item * C<or(operand)>       - makes logical `or` between object and operand
@@ -439,11 +473,156 @@ is being rendered like:
 
 =back
 
+=head1 BENCHMARKS
+
+I've compared module speed with previous abandoned implementation: L<C<Dotiac::DTL>> in both modes: FCGI and CGI. Test template and scripts are in /timethese directory.
+Django templating in Python with cache works about 80% slower than C<DTL::Fast>.
+
+=head2 FCGI/mod_perl
+
+Template parsing permormance with software cache wiping on each iteration:
+
+    Benchmark: timing 5000 iterations of DTL::Fast  , Dotiac::DTL...
+    
+    DTL::Fast  :  2 wallclock secs ( 1.31 usr +  0.75 sys =  2.06 CPU) @ 2428.36/s (n=5000)
+    Dotiac::DTL: 26 wallclock secs (11.40 usr + 13.01 sys = 24.41 CPU) @ 204.80/s (n=5000)
+
+C<DTL::Fast> parsing templates almost 12 times faster, than L<C<Dotiac::DTL>>.
+
+To run this test, you need to alter L<C<Dotiac::DTL>> module and change declaration of C<my %cache;> to C<our %cache;>. 
+    
+Rendering of pre-compiled template (software cache):
+
+    Benchmark: timing 3000 iterations of DTL::Fast  , Dotiac::DTL...
+    
+    DTL::Fast  :  7 wallclock secs ( 7.29 usr +  0.00 sys =  7.29 CPU) @ 411.81/s (n=3000)
+    Dotiac::DTL: 13 wallclock secs (12.15 usr +  0.00 sys = 12.15 CPU) @ 246.85/s (n=3000)
+
+Tests shows, that C<DTL::Fast> works 67% faster, than L<C<Dotiac::DTL>> in persistent environment.
+
+=head2 CGI
+
+This test rendered test template many times by external script, invoked via C<system> call:
+
+    Benchmark: timing 500 iterations of Dotiac render , Fast render   ...
+    
+    Dotiac render : 57 wallclock secs ( 0.16 usr +  0.44 sys =  0.59 CPU) @ 843.17/s (n=500)
+    Fast render   : 62 wallclock secs ( 0.22 usr +  0.53 sys =  0.75 CPU) @ 668.45/s (n=500)
+
+Tests shows, that C<DTL::Fast> works 26% slower, than L<C<Dotiac::DTL>> in CGI environment.
+
+=head2 DTL::Fast steps performance
+
+    1 Cache key  :  0 wallclock secs ( 0.19 usr +  0.00 sys =  0.19 CPU) @ 534759.36/s (n=100000)
+    2 Decompress :  0 wallclock secs ( 0.27 usr +  0.00 sys =  0.27 CPU) @ 377358.49/s (n=100000)
+    3 Serialize  :  4 wallclock secs ( 3.73 usr +  0.00 sys =  3.73 CPU) @ 26824.03/s (n=100000)
+    4 Deserialize:  5 wallclock secs ( 4.26 usr +  0.00 sys =  4.26 CPU) @ 23479.69/s (n=100000)
+    5 Compress   : 10 wallclock secs (10.50 usr +  0.00 sys = 10.50 CPU) @ 9524.72/s (n=100000)
+    6 Validate   : 11 wallclock secs ( 3.12 usr +  8.05 sys = 11.17 CPU) @ 8952.55/s (n=100000)
+
+    7 Parse      :  1 wallclock secs ( 0.44 usr +  0.23 sys =  0.67 CPU) @ 1492.54/s (n=1000)
+    8 Render     : 11 wallclock secs ( 9.30 usr +  1.14 sys = 10.44 CPU) @ 95.82/s (n=1000)    
+
 =head1 CHANGES
 
 =over
 
-=item * 09/01/2015 - First release, version 1.00
+=item * 18/01/2015 - v1.05 
+
+=over
+
+=item * Moved inheritance part into Template constructor 
+
+=item * Fixed bug with inheritance + blocks extension
+
+=item * Fixed bug with logic on arrays/hashes/scalars reference, object method as_bool support added. Tested.
+
+=item * Fixed bug with setting value to C<undef> using C<DTL::Fast::Context::set>
+
+=item * Fixed bug with inheritance path.
+
+=item * Implemented cache validation. Speed now is comparable to Dotiac::DTL.
+    
+=item * Made C<dirs> parameter optional for Template constructor, but it's still mandatory for C<get_template>/C<select_template>
+
+=item * Implemented cache classes: C<DTL::Fast::Cache>, C<DTL::Fast::Cache::Runtime>, C<DTL::Fast::Cache::Serialized>, C<DTL::Fast::Cache::Compressed>, C<DTL::Fast::Cache::File> and C<DTL::Fast::Cache::Memcached>. 
+
+=item * New dependencies added: L<C<Compress::Zlib>>, L<C<Digest::MD5>>
+
+=item * Added Perl::Critic testing, complies level 4.
+
+=item * Implemented C realization of spaceless tag.
+
+=back
+
+=item * 14/01/2015 - v1.04
+
+=over
+
+=item * Taken C<date> function from L<C<Dotiac::DTL>> to C<DTL::Fast::Utils::time2str_php>
+
+=item * C<now> tag and C<date> filter now works with C<time2str_php> function (like Django itself)
+
+=item * Implemented C<strftime> filter, which works with L<C<Date::Format>> C<str2time>.
+
+=item * Added Russian version of C<pluralize> filter:
+
+    use DTL::Fast;
+    use DTL::Fast::Filter::Ru::Pluralize; # this will override default pluralize with Russian version.
+
+=item * Refactored strings backup and parametrized filters.
+
+=item * C<block> and C<extends> tags now works as tags.
+
+=item * New dependency added: L<C<Storable>>
+    
+=back
+
+=item * 13/01/2015 - v1.03
+
+=over
+
+=item * Tested with CentOS & Perl 5.10
+
+=item * Lowered Perl version requirement to 5.10
+
+=item * Changed implicit split to explicit in C<wordcount> filter (v5.10 considers it depricated).
+
+=item * Added exception on missing parent template in C<extends> tag.
+
+=item * Added exception on missing included template in C<include> tag.
+
+=item * Added exception on recursive inheritance (C<extends> tag).
+
+=item * Added exception on recursive inclusion (C<include> tag).
+
+=back
+
+=item * 10/01/2015 - v1.02
+
+=over
+
+=item * changed some intermediate getters to direct access. Improved rendering performance by 10%. 
+
+=item * added tests for performance measuring and profiling (see C<timethese> directory).
+
+=back
+
+=item * 09/01/2015 - v1.01
+
+=over
+
+=item * fixed bug with C<add> filter repeated usage.
+
+=back
+
+=item * 09/01/2015 - v1.00
+
+=over
+
+=item * First release
+
+=back
 
 =back
 
