@@ -1,4 +1,4 @@
-#!/usr/bin/perl -I./lib/
+#!/usr/bin/perl -I./blib/lib -I./blib/arch
 
 use Test::Harness;
 
@@ -7,14 +7,15 @@ runtests(
     't/core/expression.t',
     't/core/loader.t',
     't/core/template.t',
+    't/expression/logic.t',
     't/expression/in.t',
+    't/expression/eq.t',
     't/expression/defined.t',
     't/filter/add.t',
     't/filter/addslashes.t',
     't/filter/capfirst.t',
     't/filter/center.t',
     't/filter/cut.t',
-    't/filter/date.t',
     't/filter/default.t',
     't/filter/default_if_none.t',
     't/filter/dictsort.t',
@@ -39,6 +40,7 @@ runtests(
     't/filter/make_list.t',
     't/filter/phone2numeric.t',
     't/filter/pluralize.t',
+    't/filter/pluralize_ru.t',
     't/filter/random.t',
     't/filter/removetags.t',
     't/filter/reverse.t',
@@ -47,6 +49,7 @@ runtests(
     't/filter/safeseq.t',
     't/filter/slice.t',
     't/filter/slugify.t',
+    't/filter/strftime.t',
     't/filter/stringformat.t',
     't/filter/striptags.t',
     't/filter/time.t',
